@@ -8,6 +8,8 @@ from pkg_resources import parse_version
 import operator
 import itertools
 
+__VERSION__ = 0.1.11
+
 class Namespace(object):
     """
     dummy class to hold namespaces during plugin loading
@@ -17,14 +19,14 @@ class Namespace(object):
 def expand_version_requierment(version):
         """
         Takes a string of one of the following forms:
-        
+
         "" -> no version requierment
         "*" -> no version requierment
         "plugin_name" -> spesfic plugin no version requierment
         "plugin_name:version_ranges" -> spesfic plugin version matches requierments
-        
+
         and returns one of the following:
-        
+
         ("", "") -> no version requierment
         ("plugin_name", "") -> plugin_name but no version requierment
         ("plugin_name", "verison_ranges")
@@ -39,13 +41,13 @@ def expand_version_requierment(version):
         else:
             return (version,  "")
 
-         
+
 
 class Plugin(object):
     """
     an object that can hold the metadata for a plugin, like its name, author, verison, and the file to be loaded ect.
     """
-    
+
     def __init__(self, config, path):
         if 'name' in config:
             self.name = config['name']
@@ -73,54 +75,54 @@ class Plugin(object):
         else:
             raise RuntimeError("Plugin as %s does not have a list provided components" % path)
         self.path = path
-        
+
     def load(self):
         filepath = os.path.join(self.path, self.file)
         spec = importlib.util.spec_from_file_location(self.name, filepath)
         plugin = spec.loader.load_module()
         return plugin
-        
+
 class System(object):
     """
     a plugin system
-    It can scan folder trees to find plugins and their provided/needed components, 
+    It can scan folder trees to find plugins and their provided/needed components,
     and with a simple load call chain load all the plugins needed.
     """
-    
+
     def __init__(self, config):
         """
         set up the system and load a configuration that may spesify plugins and versions to use for spesifc components
         plugins can define their own requerments but they are superceeded by the system configuration (carefull you can break it)
         """
-        
+
         if not isinstance(config, collections.Mapping):
             raise RuntimeError("System configurations must be mapings of component names to 'plugin:version' strings")
-        
+
         self.config = config
         self.plugins = {}
         self.components = {}
         self.loaded_components = {}
         self.loaded_plugins = {}
         self.useing = {}
-        
+
     def map_components(self, plugin_cfg):
         """
         takes a plugins meta data and remembers it's provided components so they system is awear of them
         """
         #loop through and map component names to a listing of plugin names and versions
-    
+
         for name in plugin_cfg.provides:
             # ensure a place to list component providing plugin versions
             if not name in self.components:
                 self.components[name] = {}
-            
+
             # either add the version or create a new array with the version and save it
             if plugin_cfg.name in self.components[name]:
                 self.components[name][plugin_cfg.name].append(plugin_cfg.version)
             else:
                 self.components[name][plugin_cfg.name] = [plugin_cfg.version, ]
-                
-                    
+
+
     def add_plugin(self, path):
         """
         adds a plugin form the provided path
@@ -129,9 +131,9 @@ class System(object):
         if os.path.exists(cfgpath):
             cfgfile = open(cfgpath)
             cfg = json.load(cfgfile)
-            
+
             if 'name' in cfg:
-                # ensure we have a place to map the version to the config 
+                # ensure we have a place to map the version to the config
                 if not cfg['name'] in self.plugins:
                     self.plugins[cfg['name']] = {}
                 # map the name and vserion to the config, use only the version string not the full tuple
@@ -142,7 +144,7 @@ class System(object):
                 raise RuntimeError("Plugin at %s has no name" % path)
         else:
             raise RuntimeError("No plugin exists at %s" % path)
-            
+
     def identify_plugin(self, path):
         """
         returns true if there is a plugin in the folder pointed to by path
@@ -153,7 +155,7 @@ class System(object):
         if name in names:
             return True
         return False
-        
+
     def search_dir(self, path):
         """
         recursivly searches a folder for plugins
@@ -168,19 +170,19 @@ class System(object):
                     self.add_plugin(file)
                 else:
                     self.search_dir(path)
-                
+
     def search(self, path):
         """
         search a path (folder or file) for a plugin, in the case of a file it searches the containing folder.
         """
-        # we either have a folder or a file, 
-        # if it's a file is there a plugin in the folder containing it? 
+        # we either have a folder or a file,
+        # if it's a file is there a plugin in the folder containing it?
         # if it's a folder are the plugins located somewhere within?
         if os.path.isdir(path):
             self.search_dir(path)
         else:
             self.add_plugin(os.path.dirname(path))
-            
+
     def resolve_highest_match(self, component, plugin, version):
         """
         resolves the latest version of a compoent with requierments, passing empty strings means no requierments
@@ -196,25 +198,25 @@ class System(object):
         `version1 - version2` Same as `>=version1 <=version2`.
         `range1 || range2` Passes if either range1 or range2 are satisfied.
         """
-        
+
         # we have no result yet
         result = None
-        
+
         # if we've failed to give a requierment for somthing fill it ourselves
         if plugin == "":
             # we are gettign the first plugin name in a acending alpha-numeric sort
             plugin = sorted(self.components[component])[0]
-            
+
         if version == "":
             # sort the versions for the plugin and chouse the highest one, get only the version string
             version = sorted(self.components[component][plugin], key=operator.itemgetter(1), reverse=True)[0][0]
             # if we've fallen back to the highest version we know of then there is no point veryifying it's existance, we know of it after all
             result = (plugin, version)
             return result
-            
+
         if not plugin in self.components[component]:
             raise RuntimeError("Component '%s' is not provided by 'plugin' %s" % (component, plugin))
-        
+
         # our requerments might pass if we satify one of a number of version ranges
         version_ranges = []
         if " || " in version:
@@ -222,15 +224,15 @@ class System(object):
             version_ranges = version.split(" || ")
         else:
             version_ranges.append(version)
-            
+
         # loop untill we run out of ranges to test or find a winner
         for version_range in version_ranges:
-            
+
             #markers for the high and low version of the range
             #an empty string will mean infinite and the bool indicates if the value is inclusive
-            highv =  ("", False) 
+            highv =  ("", False)
             lowv = ("", False)
-            
+
             #if the ends of the range are seperated with a dash
             if " - " in version_range:
                 # if they've tried to mix syntaxes
@@ -244,14 +246,14 @@ class System(object):
                 lowv = (high_low[1], True)
             elif " " in version_range:
                 parts = version_range.split(" ")
-                
+
                 if len(parts) != 2:
-                   raise RuntimeError("In versions useing the implicit and of a space (" ") between verison statements, there may only be 2 version statments") 
-                
+                   raise RuntimeError("In versions useing the implicit and of a space (" ") between verison statements, there may only be 2 version statments")
+
                 #they are useing implicit and, all parts must either include a > or a < +/- an =, both must be present
                 wakagreaterflag = False
                 wakaleserflag = False
-                
+
                 #we also need to establish which is the high and which is the low
                 for part in parts:
                     equalto = (part[1] == "=")
@@ -261,10 +263,10 @@ class System(object):
                     elif part[0] == "<":
                         wakaleserflag = True
                         highv = (part[1:].lstrip("="), equalto)
-                        
+
                 if not (wakagreaterflag and wakaleserflag):
                     raise RuntimeError("In versions useing the implicit and of a space (" ") between verison statements, all parts must include  either a greater or lesser-than symbolat their begining, both must be in use")
-                
+
             else:
                 # we only have one version statment
                 statment = version_range.strip().lstrip("=")
@@ -277,14 +279,14 @@ class System(object):
                     highv = (statment[1:], equalto)
                 elif statment != "*" and statment != "":
                     highv = lowv = (statment, True)
-            
+
             # now we parse the high and low versions to make them compairable and find a suitable version
             highv = (highv[0], parse_version(highv[0]), highv[1])
             lowv = (lowv[0], parse_version(lowv[0]), lowv[1])
-            
+
             # sorted from highest to lowest
             sorted_versions = sorted(self.components[component][plugin], key=operator.itemgetter(1), reverse=True)
-            
+
             #loop striping off verisons that are too high or too low
             while True:
                 stripdone = False
@@ -299,7 +301,7 @@ class System(object):
                         if sorted_versions[0][1] >= highv[1]:
                             sorted_versions = sorted_versions[1:]
                             stripdone = True
-                            
+
                 #if there is even a limit
                 if lowv[0] != "":
                     # if the low value is inclusive
@@ -311,74 +313,84 @@ class System(object):
                         if sorted_versions[len(sorted_versions) - 1][1] <= lowv[1]:
                             sorted_versions = sorted_versions[:len(sorted_versions) - 1]
                             stripdone = True
-                        
+
                 if not stripdone:
                     break
-                
+
             if len(sorted_versions) < 1:
                 raise RuntimeError("Component '%s' does not have any providers that meet requierments" % component)
-                
+
             result = (plugin, sorted_versions[0][0])
             return result
-                
-            
-    def load(self, component, requierments=None):
-        """ 
-        processes loading and returns the component by name, 
-        chain loading any required plugins to obtain dependencies. 
-        Uses the config that was provided on system creation to load correct versions, 
+
+    def ittrPluginsByComponent(self, component):
+        """
+        iterates over the all possible providers of a component returning the plugin name and the highest version avalible.
+        """
+        for name, versions in self.components[component].items():
+            version = sorted(self.components[component][name], key=operator.itemgetter(1), reverse=True)[0][0]
+            yield (name, version)
+
+    def load(self, component, requierments=None, bypass=False):
+        """
+        processes loading and returns the component by name,
+        chain loading any required plugins to obtain dependencies.
+        Uses the config that was provided on system creation to load correct versions,
         if there is a conflist throws a run time error.
+        bypass lets the call pypass the system configuration
         """
         #set default requierments
         plugin = version = plugin_req = version_req = ""
         if not component in self.components:
             raise RuntimeError("Component '%s' not provided by any loaded plugins" % component)
-        if not component in self.config:
-            warnings.warn(RuntimeWarning("Component '%s' has no default provided, defaulting to alphabetical order" % component))
-        
+
+
         # merge the systems config and the passed plugin requierments (if they were passed) to get the most relavent requierments
         reqs = {}
-        
+
         if not requierments is None:
             reqs.update(requierments)
-        reqs.update(self.config)
-        
+        if not bypass:
+            reqs.update(self.config)
+
         # update the plugin and version requierments if they exist
         if component in reqs:
             plugin_req, version_req = expand_version_requierment(reqs[component])
-            
+        else:
+            warnings.warn(RuntimeWarning("Component '%s' has no default provided, defaulting to alphabetical order" % component))
+
         # get the plugin and version to load
         plugin, version = self.resolve_highest_match(component, plugin_req, version_req)
-        
+
         # be sure not to load things twice, but besure the compoents is loaded and saved
         if not component in self.loaded_components:
             self.loaded_components[component] = {}
         if not plugin in self.loaded_components[component]:
             self.loaded_components[component][plugin] = {}
         if not version in self.loaded_components[component][plugin]:
-            
+
             #we dont want to load a plugin twice just becasue it provides more than one component, save previouly loaded plugins
             if not plugin in self.loaded_plugins:
                 self.loaded_plugins[plugin] = {}
             if not version in self.loaded_plugins[plugin]:
                 #we'll use this to store the needed components for the plugin we'll be loading
                 consumes = Namespace.__new__(Namespace)
-                
+
                 plugin_cfg = self.plugins[plugin][version]
-                
+
                 for component_req in plugin_cfg.consumes.keys():
                     setattr(consumes, component_req, self.load(component_req, plugin_cfg.consumes))
-                    
+
                 sys.modules["PyitectConsumes"] = consumes
                 self.loaded_plugins[plugin][version] = plugin_cfg.load()
                 del sys.modules["PyitectConsumes"]
-            
+
             plugin_obj = self.loaded_plugins[plugin][version]
             if not hasattr(plugin_obj, component):
                 raise RuntimeError("Plugin '%s:%s' dose not have component '%s'" % (plugin, version, component) )
-                
+
             self.loaded_components[component][plugin][version] = getattr(plugin_obj, component)
-            
+
         #record the use of this component, perhaps so the users can save the configuration
         if not component in self.useing:
             self.useing[component] = {}
@@ -386,5 +398,5 @@ class System(object):
             self.useing[component][plugin] = []
         if not version in self.useing[component][plugin]:
             self.useing[component][plugin].append(version)
-            
+
         return self.loaded_components[component][plugin][version]
